import { Component, Prop, h, State, Watch, Event } from '@stencil/core';
import { getCountAnswer } from '@boldpenguin/sdk-utility';
/**
 * This is component is only used in the legacy question grouping commercial auto flow
 */
export class BpSdkDriverQuestionSet {
  constructor() {
    /**
     * Maximum drivers allowed
     */
    this.maxDrivers = 20;
    /**
     * Mapped question groups
     */
    this.questionGroupsWithAnswersAsKey = new Map();
    this.drivers = 1;
    this.useExternal = false;
    this.addDriver = () => {
      var _a;
      const answer = getCountAnswer((_a = this.questionSet) === null || _a === void 0 ? void 0 : _a.answers);
      if (answer && this.drivers < this.maxDrivers) {
        this.saveAnswer.emit({
          answer,
          new_value: (this.drivers + 1).toString(),
        });
      }
    };
    this.deleteDriver = () => {
      var _a;
      const answer = getCountAnswer((_a = this.questionSet) === null || _a === void 0 ? void 0 : _a.answers);
      if (answer && this.drivers > 1) {
        this.saveAnswer.emit({
          answer,
          new_value: (this.drivers - 1).toString(),
        });
      }
    };
    this.handleSaveClick = (e) => {
      if (e.target['id'] === this.saveButtonId) {
        this.saveHandler(this.questionSet);
      }
    };
  }
  onQuestionSetChange(questionSet) {
    var _a, _b;
    const value = +((_b = (_a = getCountAnswer(questionSet === null || questionSet === void 0 ? void 0 : questionSet.answers)) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : 0);
    this.drivers = value;
    this.saveButtonId = `bp-sdk-save-qs-${questionSet === null || questionSet === void 0 ? void 0 : questionSet.id}`;
  }
  resetState() { }
  connectedCallback() {
    this.resetState();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-driver-question-set');
    customElements.whenDefined('bp-sdk-external-driver-question-set').then(() => {
      this.useExternal = true;
    });
    this.onQuestionSetChange(this.questionSet);
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h("bp-sdk-external-driver-question-set", { areResourcesReady: this.areResourcesReady, canSave: this.canSave, errors: this.errors, locked: this.locked, maxDrivers: this.maxDrivers, questionSet: this.questionSet, saveHandler: this.saveHandler, questionGroupsWithAnswersAsKey: this.questionGroupsWithAnswersAsKey }));
  }
  renderInternal() {
    return (h("div", { class: "bp-sdk-driver-question-set__container" },
      h("bp-sdk-add-item-container", { canAddMore: this.drivers && this.drivers < this.maxDrivers, onAddItem: this.addDriver, helpText: "Have additional Drivers?", text: "Add driver" }, [...this.questionGroupsWithAnswersAsKey].map(([_, group]) => {
        return h("bp-sdk-driver-group", { group: group, errors: this.errors, canDelete: this.drivers > 1, onDeleteItem: this.deleteDriver });
      })),
      h("bp-sdk-driver-question-set-button", { text: "Save Question Set", buttonId: this.saveButtonId, buttonDisabled: !this.areResourcesReady || !this.canSave, onClick: this.handleSaveClick })));
  }
  static get is() { return "bp-sdk-driver-question-set"; }
  static get originalStyleUrls() { return {
    "$": ["./bp-sdk-driver-question-set.css"]
  }; }
  static get styleUrls() { return {
    "$": ["bp-sdk-driver-question-set.css"]
  }; }
  static get properties() { return {
    "questionSet": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "IApplicationFormQuestionSet",
        "resolved": "IApplicationFormQuestionSet",
        "references": {
          "IApplicationFormQuestionSet": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question set"
      }
    },
    "errors": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "BpSdkStore['application']['errors']",
        "resolved": "IErrorState",
        "references": {
          "BpSdkStore": {
            "location": "import",
            "path": "@boldpenguin/sdk-redux"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Error states"
      }
    },
    "maxDrivers": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Maximum drivers allowed"
      },
      "attribute": "max-drivers",
      "reflect": false,
      "defaultValue": "20"
    },
    "areResourcesReady": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Flag used to disable button if resources for question set are not ready"
      },
      "attribute": "are-resources-ready",
      "reflect": false
    },
    "locked": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Flag used to disable all answer inputs"
      },
      "attribute": "locked",
      "reflect": false
    },
    "canSave": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Can Save"
      },
      "attribute": "can-save",
      "reflect": false
    },
    "saveHandler": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "(questionSet: IApplicationFormQuestionSet) => void",
        "resolved": "(questionSet: IApplicationFormQuestionSet) => void",
        "references": {
          "IApplicationFormQuestionSet": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Save Question set function"
      }
    },
    "questionGroupsWithAnswersAsKey": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Map<string, IMappedQuestionGroupAnswers>",
        "resolved": "Map<string, IMappedQuestionGroupAnswers>",
        "references": {
          "Map": {
            "location": "global"
          },
          "IMappedQuestionGroupAnswers": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Mapped question groups"
      },
      "defaultValue": "new Map()"
    }
  }; }
  static get states() { return {
    "drivers": {},
    "useExternal": {},
    "saveButtonId": {}
  }; }
  static get events() { return [{
      "method": "saveAnswer",
      "name": "saveAnswer",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Passes information needed to update answer"
      },
      "complexType": {
        "original": "{\n    answer: IAnswer;\n    new_value: string;\n  }",
        "resolved": "{ answer: IAnswer; new_value: string; }",
        "references": {
          "IAnswer": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      }
    }]; }
  static get watchers() { return [{
      "propName": "questionSet",
      "methodName": "onQuestionSetChange"
    }]; }
}
