import { doesUserOwnApplicationForm, getAnswersState, getApplicationFormState, getCurrentActiveQuestionSet, getPartnerEngineConfig, getQuestionPoolState, getQuestionSetState, removeActiveAnswerId, saveQuestionSet, selectQuestionSetsState, switchActiveQuestionSet, pushAnswer, updateErrors, userHasPermission, } from '@boldpenguin/sdk-redux';
import { AnalyticsTrackingEventName, ApplicationFormProcessState, BpSdkPermissionAction, BpSdkPermissions, LoadingStates, } from '@boldpenguin/sdk-types';
import { areAnswersValid, containsOnlyQuestionPoolAnchorQuestions, focusErrors, getErrors, getParentAnchorAnswer, isValidAnswer, calculateParentAnchorValue, updateAnswerValue } from '@boldpenguin/sdk-utility';
import { Component, Event, h, Listen, Prop, State, Watch } from '@stencil/core';
import { store } from '@stencil/redux';
import { canGetStore } from '../../utils/store';
export class BpSdkNextButtonContainer {
  constructor() {
    /**
     * Controls disabled state of button
     */
    this.buttonDisabled = false;
    /**
     * Disable navigation if active question set is incomplete
     */
    this.shouldDisableIfIncomplete = false;
    this.useExternal = false;
    this.canSave = true;
    this.areResourcesReady = true;
    this.isDisabled = false;
    this.caDisabled = false;
    this.isReady = true;
    this.isLocked = false;
    this.isFirstRun = true;
    this.classes = '';
    this.handleSubscriptions = () => {
      if (canGetStore()) {
        if (this.storeUnsubscribe) {
          this.storeUnsubscribe();
        }
        store.mapDispatchToProps(this, {
          saveQuestionSet,
          updateErrors,
          removeActiveAnswerId,
          switchActiveQuestionSet,
          pushAnswer,
        });
        this.storeUnsubscribe = store.mapStateToProps(this, this.mapStateToProps);
      }
    };
    this.mapStateToProps = (state) => ({
      activeQuestionSet: getCurrentActiveQuestionSet(state),
      config: getPartnerEngineConfig(state),
      answersResourceState: getAnswersState(state),
      applicationFormResourceState: getApplicationFormState(state),
      pendingProcesses: state.application.pendingProcesses,
      questionPoolResourceState: getQuestionPoolState(state),
      questionSetResourceState: getQuestionSetState(state),
      questionSetState: selectQuestionSetsState(state),
      queue: state.queues.answers.queue,
      enableUserApplicationGuard: !!state.config.options.enableUserApplicationGuard,
      locked: state.application.form.locked,
      isOwner: doesUserOwnApplicationForm(state),
      hasPermissionToEdit: userHasPermission(state, {
        permission: BpSdkPermissions.ApplicationForms,
        action: BpSdkPermissionAction.Update,
      }),
    });
  }
  resetState() {
    this.isReady = true;
    this.isFirstRun = true;
  }
  connectedCallback() {
    this.resetState();
    this.handleSubscriptions();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-next-button-container');
    customElements.whenDefined('bp-sdk-external-next-button-container').then(() => {
      this.useExternal = true;
    });
    this.handleSubscriptions();
    this.onShouldDisableIfIncompleteChange(this.shouldDisableIfIncomplete);
    this.onAnswersResourcesStateChange(this.answersResourceState);
    this.onApplicationFormResourceStateChange(this.applicationFormResourceState);
    this.onHasPermissionToEdit(this.hasPermissionToEdit);
    this.onEnableUserApplicationGuard(this.enableUserApplicationGuard);
    this.onIsFirstRun(this.isFirstRun);
    this.onIsOwner(this.isOwner);
    this.onIsReady(this.isReady);
    this.onLocked(this.locked);
    this.onQuestionPoolResourcesStateChange(this.questionPoolResourceState);
    this.onQuestionSetResourcesStateChange(this.questionSetResourceState);
    this.onQueueChange(this.queue);
  }
  onEnableUserApplicationGuard(enableUserApplicationGuard) {
    this.updateIsLocked(this.locked, this.isOwner, this.hasPermissionToEdit, enableUserApplicationGuard);
  }
  onHasPermissionToEdit(hasPermission) {
    this.updateIsLocked(this.locked, this.isOwner, hasPermission, this.enableUserApplicationGuard);
  }
  onIsOwner(isOwner) {
    this.updateIsLocked(this.locked, isOwner, this.hasPermissionToEdit, this.enableUserApplicationGuard);
  }
  onLocked(locked) {
    this.updateIsLocked(locked, this.isOwner, this.hasPermissionToEdit, this.enableUserApplicationGuard);
  }
  updateIsLocked(locked, isOwner, hasPermission, enableUserApplicationGuard) {
    this.isLocked = locked || (enableUserApplicationGuard && (!isOwner || !hasPermission));
  }
  onIsFirstRun(isFirstRun) {
    this.updateAreResourcesReady(isFirstRun, this.isReady);
  }
  onIsReady(isReady) {
    this.updateAreResourcesReady(this.isFirstRun, isReady);
  }
  updateAreResourcesReady(isFirstRun, isReady) {
    this.areResourcesReady = isFirstRun || isReady;
  }
  onAnswersResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: state,
      applicationForm: this.applicationFormResourceState,
      questionPools: this.questionPoolResourceState,
      questionSets: this.questionSetResourceState,
    });
  }
  onApplicationFormResourceStateChange(state) {
    if (this.isFirstRun && state === LoadingStates.Loaded) {
      this.isFirstRun = false;
    }
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: state,
      questionPools: this.questionPoolResourceState,
      questionSets: this.questionSetResourceState,
    });
  }
  onQuestionPoolResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: this.applicationFormResourceState,
      questionPools: state,
      questionSets: this.questionSetResourceState,
    });
  }
  onQuestionSetResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: this.applicationFormResourceState,
      questionPools: this.questionPoolResourceState,
      questionSets: state,
    });
  }
  calculateResourcesReady(resources = {}) {
    if (!resources ||
      resources.applicationForm !== LoadingStates.Loaded ||
      resources.answers === LoadingStates.Loading ||
      resources.questionPools === LoadingStates.Loading ||
      resources.questionSets === LoadingStates.Loading) {
      return false;
    }
    return true;
  }
  onShouldDisableIfIncompleteChange(shouldDisableIfIncomplete) {
    this.updateOnDisableIfIncomplete(shouldDisableIfIncomplete, this.activeQuestionSet);
  }
  onActiveQuestionSetChange(activeQuestionSet) {
    this.updateOnDisableIfIncomplete(this.shouldDisableIfIncomplete, activeQuestionSet);
    this.disableIfCAQuestionSet(this.shouldDisableIfIncomplete, activeQuestionSet);
  }
  onQueueChange(queue) {
    this.canSave = !(queue === null || queue === void 0 ? void 0 : queue.length) && !this.doesCurrentQuestionSetHaveAPendingProcess();
  }
  onPendingProcessesChange(processes) {
    this.canSave = !this.doesCurrentQuestionSetHaveAPendingProcess(processes);
  }
  disableIfCAQuestionSet(shouldDisableIfIncomplete, activeQuestionSet) {
    let shouldDisable = false;
    if (shouldDisableIfIncomplete && activeQuestionSet) {
      shouldDisable = containsOnlyQuestionPoolAnchorQuestions(activeQuestionSet);
    }
    if (this.caDisabled !== shouldDisable) {
      this.caDisabled = shouldDisable;
    }
  }
  updateOnDisableIfIncomplete(shouldDisableIfIncomplete, activeQuestionSet) {
    this.isDisabled =
      shouldDisableIfIncomplete &&
        (!activeQuestionSet ||
          !activeQuestionSet.answers ||
          !activeQuestionSet.answers.length ||
          activeQuestionSet.answers.some((answer) => !answer.is_completed && answer.required));
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h("bp-sdk-external-next-button-container", { activeQuestionSet: this.activeQuestionSet, answersResourceState: this.answersResourceState, applicationFormResourceState: this.applicationFormResourceState, buttonDisabled: this.buttonDisabled, config: this.config, enableUserApplicationGuard: this.enableUserApplicationGuard, hasPermissionToEdit: this.hasPermissionToEdit, isOwner: this.isOwner, locked: this.locked, pendingProcesses: this.pendingProcesses, questionPoolResourceState: this.questionPoolResourceState, questionSetResourceState: this.questionSetResourceState, questionSetState: this.questionSetState, queue: this.queue, shouldDisableIfIncomplete: this.shouldDisableIfIncomplete }));
  }
  handleSaveQuestionSet() {
    if (!this.isLocked) {
      const invalidAnswers = this.activeQuestionSet.answers.filter((answer) => !isValidAnswer(answer));
      if (!invalidAnswers.length) {
        this.removeActiveAnswerId();
        if (!this.activeQuestionSet.parentQuestionSetId) {
          this.callSaveQuestionSet();
        }
        else {
          if (this.parentQuestionSetCanSave()) {
            this.callSaveQuestionSet(true);
          }
          else {
            this.handleStep();
          }
        }
      }
      else {
        invalidAnswers.forEach((answer) => {
          this.updateErrors({ answer, errors: getErrors(answer) });
        });
        focusErrors();
      }
    }
    else {
      this.switchActiveQuestionSet('forward');
    }
  }
  renderInternal() {
    return (h("bp-sdk-next-button", { buttonDisabled: this.buttonDisabled || !this.areResourcesReady || !this.canSave || this.isDisabled || this.caDisabled, cssClasses: !this.areResourcesReady || !this.canSave ? 'loading' : 'not-loading' }));
  }
  doesCurrentQuestionSetHaveAPendingProcess(processes) {
    const currentProcesses = processes ? processes : this.pendingProcesses;
    if (!(currentProcesses === null || currentProcesses === void 0 ? void 0 : currentProcesses.length)) {
      return false;
    }
    return currentProcesses.some((process) => process.state !== ApplicationFormProcessState.Completed && process.question_set_id === this.activeQuestionSet.id);
  }
  parentQuestionSetCanSave() {
    const parentQuestionSet = this.questionSetState.parentQuestionSets.find((pqs) => pqs.id === this.activeQuestionSet.parentQuestionSetId);
    if (parentQuestionSet) {
      return this.questionSetState.list
        .filter((qs) => qs.parentQuestionSetId === parentQuestionSet.id)
        .every((qs) => areAnswersValid(qs.answers));
    }
  }
  callSaveQuestionSet(useParent = false) {
    this.saveQuestionSet({
      config: this.config,
      appFormQuestionSet: this.activeQuestionSet,
      navigationDirection: 'forward',
      parentQuestionSet: useParent
        ? this.questionSetState.parentQuestionSets.find((pqs) => pqs.id === this.activeQuestionSet.parentQuestionSetId)
        : undefined,
    });
    this.bpAnalyticsTrack.emit({
      eventName: AnalyticsTrackingEventName.QuestionSetSaved,
      eventPayload: {
        applicationFormQuestionSetId: this.activeQuestionSet.id,
        questionSetId: this.activeQuestionSet.question_set_id,
        questionSetCode: this.activeQuestionSet.question_set.code,
      },
    });
  }
  handleStep() {
    const parent = getParentAnchorAnswer(this.activeQuestionSet, this.questionSetState);
    const newValue = calculateParentAnchorValue(this.activeQuestionSet);
    if (parent.value !== newValue) {
      this.pushAnswer(updateAnswerValue(parent, newValue));
    }
    this.switchActiveQuestionSet('forward');
  }
  static get is() { return "bp-sdk-next-button-container"; }
  static get originalStyleUrls() { return {
    "$": ["./bp-sdk-next-button-container.css"]
  }; }
  static get styleUrls() { return {
    "$": ["bp-sdk-next-button-container.css"]
  }; }
  static get properties() { return {
    "activeQuestionSet": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IApplicationFormQuestionSet",
        "resolved": "IApplicationFormQuestionSet",
        "references": {
          "IApplicationFormQuestionSet": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    },
    "buttonDisabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Controls disabled state of button"
      },
      "attribute": "button-disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "config": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IPartnerEngineConfig",
        "resolved": "IPartnerEngineConfig",
        "references": {
          "IPartnerEngineConfig": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "IPartnerEngineConfig injected from redux"
      }
    },
    "queue": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IAnswer[]",
        "resolved": "IAnswer[]",
        "references": {
          "IAnswer": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Answer queue"
      }
    },
    "shouldDisableIfIncomplete": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disable navigation if active question set is incomplete"
      },
      "attribute": "should-disable-if-incomplete",
      "reflect": false,
      "defaultValue": "false"
    },
    "answersResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Answer resource states injected from redux"
      },
      "attribute": "answers-resource-state",
      "reflect": false
    },
    "applicationFormResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Application form resource states injected from redux"
      },
      "attribute": "application-form-resource-state",
      "reflect": false
    },
    "questionPoolResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question pool resource states injected from redux"
      },
      "attribute": "question-pool-resource-state",
      "reflect": false
    },
    "questionSetResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question set resource states injected from redux"
      },
      "attribute": "question-set-resource-state",
      "reflect": false
    },
    "questionSetState": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "IQuestionSetState",
        "resolved": "IQuestionSetState",
        "references": {
          "IQuestionSetState": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question Set State from Store"
      }
    },
    "locked": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Application form state injected from redux"
      },
      "attribute": "locked",
      "reflect": false
    },
    "pendingProcesses": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IApplicationFormProcesses[]",
        "resolved": "IApplicationFormProcesses[]",
        "references": {
          "IApplicationFormProcesses": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Pending processes of application form state injected from redux"
      }
    },
    "isOwner": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Is owner of application"
      },
      "attribute": "is-owner",
      "reflect": false
    },
    "hasPermissionToEdit": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Has permission to edit application"
      },
      "attribute": "has-permission-to-edit",
      "reflect": false
    },
    "enableUserApplicationGuard": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enabled user check for application"
      },
      "attribute": "enable-user-application-guard",
      "reflect": false
    }
  }; }
  static get states() { return {
    "useExternal": {},
    "canSave": {},
    "areResourcesReady": {},
    "isDisabled": {},
    "caDisabled": {},
    "isReady": {},
    "isLocked": {},
    "isFirstRun": {},
    "classes": {}
  }; }
  static get events() { return [{
      "method": "bpAnalyticsTrack",
      "name": "bpAnalyticsTrack",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an analytics tracking event, see `AnalyticsTrackingEventName` (sdk-types) for event names"
      },
      "complexType": {
        "original": "IAnalyticsTrackEventDetail",
        "resolved": "IAnalyticsTrackEventDetail",
        "references": {
          "IAnalyticsTrackEventDetail": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      }
    }]; }
  static get watchers() { return [{
      "propName": "enableUserApplicationGuard",
      "methodName": "onEnableUserApplicationGuard"
    }, {
      "propName": "hasPermissionToEdit",
      "methodName": "onHasPermissionToEdit"
    }, {
      "propName": "isOwner",
      "methodName": "onIsOwner"
    }, {
      "propName": "locked",
      "methodName": "onLocked"
    }, {
      "propName": "isFirstRun",
      "methodName": "onIsFirstRun"
    }, {
      "propName": "isReady",
      "methodName": "onIsReady"
    }, {
      "propName": "answersResourceState",
      "methodName": "onAnswersResourcesStateChange"
    }, {
      "propName": "applicationFormResourceState",
      "methodName": "onApplicationFormResourceStateChange"
    }, {
      "propName": "questionPoolResourceState",
      "methodName": "onQuestionPoolResourcesStateChange"
    }, {
      "propName": "questionSetResourceState",
      "methodName": "onQuestionSetResourcesStateChange"
    }, {
      "propName": "shouldDisableIfIncomplete",
      "methodName": "onShouldDisableIfIncompleteChange"
    }, {
      "propName": "activeQuestionSet",
      "methodName": "onActiveQuestionSetChange"
    }, {
      "propName": "queue",
      "methodName": "onQueueChange"
    }, {
      "propName": "pendingProcesses",
      "methodName": "onPendingProcessesChange"
    }]; }
  static get listeners() { return [{
      "name": "navigateNext",
      "method": "handleSaveQuestionSet",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
