import { getCountAnswer, getIsTrailerQuestionGroup, getVehicleAnswers } from '@boldpenguin/sdk-utility';
import { Component, Event, h, Prop, State, Watch } from '@stencil/core';
/**
 * This is component is only used in the legacy question grouping commercial auto flow
 */
export class BpSdkVehicleQuestionSet {
  constructor() {
    /**
     * Maximum vehicles allowed
     */
    this.maxVehicles = 20;
    /**
     * A Map of nested question groups for rendering nested question groups/question pools
     */
    this.questionGroupsWithAnswersAsKey = new Map();
    this.vehicles = 1;
    this.useExternal = false;
    this.canDelete = (index) => {
      return index > 0 && index === this.vehicles - 1;
    };
    this.addVehicle = () => {
      var _a;
      const answer = getCountAnswer((_a = this.questionSet) === null || _a === void 0 ? void 0 : _a.answers);
      if (answer && this.vehicles < this.maxVehicles) {
        this.saveAnswer.emit({
          answer,
          new_value: (this.vehicles + 1).toString(),
        });
      }
    };
    this.deleteVehicle = () => {
      var _a;
      const answer = getCountAnswer((_a = this.questionSet) === null || _a === void 0 ? void 0 : _a.answers);
      if (answer && this.vehicles > 1) {
        this.saveAnswer.emit({
          answer,
          new_value: (this.vehicles - 1).toString(),
        });
      }
    };
    this.handleSaveClick = (e) => {
      if (this.useExternal) {
        return;
      }
      if (e.target['id'] === this.saveButtonId) {
        this.saveHandler(this.questionSet);
      }
    };
  }
  buildExtraGroupInformation() {
    return [...this.questionGroupsWithAnswersAsKey].reduce((acc, [key, mappedGroup]) => (Object.assign(Object.assign({}, acc), { [key]: {
        answers: getVehicleAnswers(mappedGroup),
      } })), {});
  }
  onQuestionSetChange(questionSet) {
    var _a, _b;
    const value = +((_b = (_a = getCountAnswer(questionSet === null || questionSet === void 0 ? void 0 : questionSet.answers)) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : 0);
    this.vehicles = value;
    this.saveButtonId = `bp-sdk-save-qs-${questionSet === null || questionSet === void 0 ? void 0 : questionSet.id}`;
  }
  onQuestionGroupsWithAnswersAsKeyChange(_) {
    this.extraGroupInformationMap = this.buildExtraGroupInformation();
  }
  resetState() { }
  connectedCallback() {
    this.resetState();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-vehicle-question-set');
    customElements.whenDefined('bp-sdk-external-vehicle-question-set').then(() => {
      this.useExternal = true;
    });
    this.onQuestionSetChange(this.questionSet);
    this.onQuestionGroupsWithAnswersAsKeyChange(this.questionGroupsWithAnswersAsKey);
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h("bp-sdk-external-vehicle-question-set", { areResourcesReady: this.areResourcesReady, canSave: this.canSave, locked: this.locked, maxVehicles: this.maxVehicles, questionSet: this.questionSet, saveHandler: this.saveHandler, questionGroupsWithAnswersAsKey: this.questionGroupsWithAnswersAsKey }));
  }
  renderInternal() {
    return (h("fieldset", { disabled: this.locked, class: "bp-sdk-vehicle-question-set__container" },
      h("bp-sdk-add-item-container", { canAddMore: this.vehicles && this.vehicles < this.maxVehicles, onAddItem: this.addVehicle, helpText: "Have additional Vehicles?", text: "Add vehicle" }, [...this.questionGroupsWithAnswersAsKey].map(([key, group], index) => {
        return (h("bp-sdk-vehicle-panel-container", { onDeleteItem: this.deleteVehicle, questionGroup: group, canDelete: this.canDelete(index), shouldShowTrailers: index >= 1 || getIsTrailerQuestionGroup(group), isVehicleTrailer: getIsTrailerQuestionGroup(group), answers: this.extraGroupInformationMap[key].answers, code: group.parentQuestionAnswer.question.id }));
      })),
      h("bp-sdk-vehicle-question-set-button", { text: "Save Question Set", buttonId: this.saveButtonId, buttonDisabled: !this.areResourcesReady || !this.canSave, extraCssClasses: !this.areResourcesReady || !this.canSave ? 'loading' : 'not-loading', onClick: this.handleSaveClick })));
  }
  static get is() { return "bp-sdk-vehicle-question-set"; }
  static get originalStyleUrls() { return {
    "$": ["./bp-sdk-vehicle-question-set.css"]
  }; }
  static get styleUrls() { return {
    "$": ["bp-sdk-vehicle-question-set.css"]
  }; }
  static get properties() { return {
    "saveHandler": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "(questionSet: IApplicationFormQuestionSet) => void",
        "resolved": "(questionSet: IApplicationFormQuestionSet) => void",
        "references": {
          "IApplicationFormQuestionSet": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Save Question set function"
      }
    },
    "questionSet": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "IApplicationFormQuestionSet",
        "resolved": "IApplicationFormQuestionSet",
        "references": {
          "IApplicationFormQuestionSet": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question set"
      }
    },
    "maxVehicles": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Maximum vehicles allowed"
      },
      "attribute": "max-vehicles",
      "reflect": false,
      "defaultValue": "20"
    },
    "areResourcesReady": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Flag used to disable button if resources for question set are not ready"
      },
      "attribute": "are-resources-ready",
      "reflect": false
    },
    "locked": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Flag used to disable all answer inputs"
      },
      "attribute": "locked",
      "reflect": false
    },
    "canSave": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Can Save"
      },
      "attribute": "can-save",
      "reflect": false
    },
    "questionGroupsWithAnswersAsKey": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Map<string, IMappedQuestionGroupAnswers>",
        "resolved": "Map<string, IMappedQuestionGroupAnswers>",
        "references": {
          "Map": {
            "location": "global"
          },
          "IMappedQuestionGroupAnswers": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "A Map of nested question groups for rendering nested question groups/question pools"
      },
      "defaultValue": "new Map()"
    }
  }; }
  static get states() { return {
    "vehicles": {},
    "useExternal": {},
    "saveButtonId": {},
    "extraGroupInformationMap": {}
  }; }
  static get events() { return [{
      "method": "saveAnswer",
      "name": "saveAnswer",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Passes information needed to update answer"
      },
      "complexType": {
        "original": "{\n    answer: IAnswer;\n    new_value: string;\n  }",
        "resolved": "{ answer: IAnswer; new_value: string; }",
        "references": {
          "IAnswer": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      }
    }]; }
  static get watchers() { return [{
      "propName": "questionSet",
      "methodName": "onQuestionSetChange"
    }, {
      "propName": "questionGroupsWithAnswersAsKey",
      "methodName": "onQuestionGroupsWithAnswersAsKeyChange"
    }]; }
}
