import { r as registerInstance, c as createEvent, h as h$1 } from './index-b4e0290f.js';
import { saveQuestionSet, updateErrors, removeActiveAnswerId, switchActiveQuestionSet, pushAnswer, getCurrentActiveQuestionSet, getPartnerEngineConfig, getAnswersState, getApplicationFormState, getQuestionPoolState, getQuestionSetState, selectQuestionSetsState, doesUserOwnApplicationForm, userHasPermission } from '@boldpenguin/sdk-redux';
import { v, h, i, k, o as oe } from './index.esm.min-5e144817.js';
import { g as gc, p as pc, h as ha, v as vx, a as va, f as f4, l as l4, b as bl } from './index.esm.min-a0c9e6cc.js';
import { c as canGetStore, s as store } from './store-6dda1ad2.js';

const bpSdkNextButtonContainerCss = "";

let BpSdkNextButtonContainer = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.bpAnalyticsTrack = createEvent(this, "bpAnalyticsTrack", 7);
    /**
     * Controls disabled state of button
     */
    this.buttonDisabled = false;
    /**
     * Disable navigation if active question set is incomplete
     */
    this.shouldDisableIfIncomplete = false;
    this.useExternal = false;
    this.canSave = true;
    this.areResourcesReady = true;
    this.isDisabled = false;
    this.caDisabled = false;
    this.isReady = true;
    this.isLocked = false;
    this.isFirstRun = true;
    this.classes = '';
    this.handleSubscriptions = () => {
      if (canGetStore()) {
        if (this.storeUnsubscribe) {
          this.storeUnsubscribe();
        }
        store.mapDispatchToProps(this, {
          saveQuestionSet,
          updateErrors,
          removeActiveAnswerId,
          switchActiveQuestionSet,
          pushAnswer,
        });
        this.storeUnsubscribe = store.mapStateToProps(this, this.mapStateToProps);
      }
    };
    this.mapStateToProps = (state) => ({
      activeQuestionSet: getCurrentActiveQuestionSet(state),
      config: getPartnerEngineConfig(state),
      answersResourceState: getAnswersState(state),
      applicationFormResourceState: getApplicationFormState(state),
      pendingProcesses: state.application.pendingProcesses,
      questionPoolResourceState: getQuestionPoolState(state),
      questionSetResourceState: getQuestionSetState(state),
      questionSetState: selectQuestionSetsState(state),
      queue: state.queues.answers.queue,
      enableUserApplicationGuard: !!state.config.options.enableUserApplicationGuard,
      locked: state.application.form.locked,
      isOwner: doesUserOwnApplicationForm(state),
      hasPermissionToEdit: userHasPermission(state, {
        permission: v.ApplicationForms,
        action: h.Update,
      }),
    });
  }
  resetState() {
    this.isReady = true;
    this.isFirstRun = true;
  }
  connectedCallback() {
    this.resetState();
    this.handleSubscriptions();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-next-button-container');
    customElements.whenDefined('bp-sdk-external-next-button-container').then(() => {
      this.useExternal = true;
    });
    this.handleSubscriptions();
    this.onShouldDisableIfIncompleteChange(this.shouldDisableIfIncomplete);
    this.onAnswersResourcesStateChange(this.answersResourceState);
    this.onApplicationFormResourceStateChange(this.applicationFormResourceState);
    this.onHasPermissionToEdit(this.hasPermissionToEdit);
    this.onEnableUserApplicationGuard(this.enableUserApplicationGuard);
    this.onIsFirstRun(this.isFirstRun);
    this.onIsOwner(this.isOwner);
    this.onIsReady(this.isReady);
    this.onLocked(this.locked);
    this.onQuestionPoolResourcesStateChange(this.questionPoolResourceState);
    this.onQuestionSetResourcesStateChange(this.questionSetResourceState);
    this.onQueueChange(this.queue);
  }
  onEnableUserApplicationGuard(enableUserApplicationGuard) {
    this.updateIsLocked(this.locked, this.isOwner, this.hasPermissionToEdit, enableUserApplicationGuard);
  }
  onHasPermissionToEdit(hasPermission) {
    this.updateIsLocked(this.locked, this.isOwner, hasPermission, this.enableUserApplicationGuard);
  }
  onIsOwner(isOwner) {
    this.updateIsLocked(this.locked, isOwner, this.hasPermissionToEdit, this.enableUserApplicationGuard);
  }
  onLocked(locked) {
    this.updateIsLocked(locked, this.isOwner, this.hasPermissionToEdit, this.enableUserApplicationGuard);
  }
  updateIsLocked(locked, isOwner, hasPermission, enableUserApplicationGuard) {
    this.isLocked = locked || (enableUserApplicationGuard && (!isOwner || !hasPermission));
  }
  onIsFirstRun(isFirstRun) {
    this.updateAreResourcesReady(isFirstRun, this.isReady);
  }
  onIsReady(isReady) {
    this.updateAreResourcesReady(this.isFirstRun, isReady);
  }
  updateAreResourcesReady(isFirstRun, isReady) {
    this.areResourcesReady = isFirstRun || isReady;
  }
  onAnswersResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: state,
      applicationForm: this.applicationFormResourceState,
      questionPools: this.questionPoolResourceState,
      questionSets: this.questionSetResourceState,
    });
  }
  onApplicationFormResourceStateChange(state) {
    if (this.isFirstRun && state === i.Loaded) {
      this.isFirstRun = false;
    }
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: state,
      questionPools: this.questionPoolResourceState,
      questionSets: this.questionSetResourceState,
    });
  }
  onQuestionPoolResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: this.applicationFormResourceState,
      questionPools: state,
      questionSets: this.questionSetResourceState,
    });
  }
  onQuestionSetResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: this.applicationFormResourceState,
      questionPools: this.questionPoolResourceState,
      questionSets: state,
    });
  }
  calculateResourcesReady(resources = {}) {
    if (!resources ||
      resources.applicationForm !== i.Loaded ||
      resources.answers === i.Loading ||
      resources.questionPools === i.Loading ||
      resources.questionSets === i.Loading) {
      return false;
    }
    return true;
  }
  onShouldDisableIfIncompleteChange(shouldDisableIfIncomplete) {
    this.updateOnDisableIfIncomplete(shouldDisableIfIncomplete, this.activeQuestionSet);
  }
  onActiveQuestionSetChange(activeQuestionSet) {
    this.updateOnDisableIfIncomplete(this.shouldDisableIfIncomplete, activeQuestionSet);
    this.disableIfCAQuestionSet(this.shouldDisableIfIncomplete, activeQuestionSet);
  }
  onQueueChange(queue) {
    this.canSave = !(queue === null || queue === void 0 ? void 0 : queue.length) && !this.doesCurrentQuestionSetHaveAPendingProcess();
  }
  onPendingProcessesChange(processes) {
    this.canSave = !this.doesCurrentQuestionSetHaveAPendingProcess(processes);
  }
  disableIfCAQuestionSet(shouldDisableIfIncomplete, activeQuestionSet) {
    let shouldDisable = false;
    if (shouldDisableIfIncomplete && activeQuestionSet) {
      shouldDisable = gc(activeQuestionSet);
    }
    if (this.caDisabled !== shouldDisable) {
      this.caDisabled = shouldDisable;
    }
  }
  updateOnDisableIfIncomplete(shouldDisableIfIncomplete, activeQuestionSet) {
    this.isDisabled =
      shouldDisableIfIncomplete &&
        (!activeQuestionSet ||
          !activeQuestionSet.answers ||
          !activeQuestionSet.answers.length ||
          activeQuestionSet.answers.some((answer) => !answer.is_completed && answer.required));
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h$1("bp-sdk-external-next-button-container", { activeQuestionSet: this.activeQuestionSet, answersResourceState: this.answersResourceState, applicationFormResourceState: this.applicationFormResourceState, buttonDisabled: this.buttonDisabled, config: this.config, enableUserApplicationGuard: this.enableUserApplicationGuard, hasPermissionToEdit: this.hasPermissionToEdit, isOwner: this.isOwner, locked: this.locked, pendingProcesses: this.pendingProcesses, questionPoolResourceState: this.questionPoolResourceState, questionSetResourceState: this.questionSetResourceState, questionSetState: this.questionSetState, queue: this.queue, shouldDisableIfIncomplete: this.shouldDisableIfIncomplete }));
  }
  handleSaveQuestionSet() {
    if (!this.isLocked) {
      const invalidAnswers = this.activeQuestionSet.answers.filter((answer) => !pc(answer));
      if (!invalidAnswers.length) {
        this.removeActiveAnswerId();
        if (!this.activeQuestionSet.parentQuestionSetId) {
          this.callSaveQuestionSet();
        }
        else {
          if (this.parentQuestionSetCanSave()) {
            this.callSaveQuestionSet(true);
          }
          else {
            this.handleStep();
          }
        }
      }
      else {
        invalidAnswers.forEach((answer) => {
          this.updateErrors({ answer, errors: ha(answer) });
        });
        vx();
      }
    }
    else {
      this.switchActiveQuestionSet('forward');
    }
  }
  renderInternal() {
    return (h$1("bp-sdk-next-button", { buttonDisabled: this.buttonDisabled || !this.areResourcesReady || !this.canSave || this.isDisabled || this.caDisabled, cssClasses: !this.areResourcesReady || !this.canSave ? 'loading' : 'not-loading' }));
  }
  doesCurrentQuestionSetHaveAPendingProcess(processes) {
    const currentProcesses = processes ? processes : this.pendingProcesses;
    if (!(currentProcesses === null || currentProcesses === void 0 ? void 0 : currentProcesses.length)) {
      return false;
    }
    return currentProcesses.some((process) => process.state !== k.Completed && process.question_set_id === this.activeQuestionSet.id);
  }
  parentQuestionSetCanSave() {
    const parentQuestionSet = this.questionSetState.parentQuestionSets.find((pqs) => pqs.id === this.activeQuestionSet.parentQuestionSetId);
    if (parentQuestionSet) {
      return this.questionSetState.list
        .filter((qs) => qs.parentQuestionSetId === parentQuestionSet.id)
        .every((qs) => va(qs.answers));
    }
  }
  callSaveQuestionSet(useParent = false) {
    this.saveQuestionSet({
      config: this.config,
      appFormQuestionSet: this.activeQuestionSet,
      navigationDirection: 'forward',
      parentQuestionSet: useParent
        ? this.questionSetState.parentQuestionSets.find((pqs) => pqs.id === this.activeQuestionSet.parentQuestionSetId)
        : undefined,
    });
    this.bpAnalyticsTrack.emit({
      eventName: oe.QuestionSetSaved,
      eventPayload: {
        applicationFormQuestionSetId: this.activeQuestionSet.id,
        questionSetId: this.activeQuestionSet.question_set_id,
        questionSetCode: this.activeQuestionSet.question_set.code,
      },
    });
  }
  handleStep() {
    const parent = f4(this.activeQuestionSet, this.questionSetState);
    const newValue = l4(this.activeQuestionSet);
    if (parent.value !== newValue) {
      this.pushAnswer(bl(parent, newValue));
    }
    this.switchActiveQuestionSet('forward');
  }
  static get watchers() { return {
    "enableUserApplicationGuard": ["onEnableUserApplicationGuard"],
    "hasPermissionToEdit": ["onHasPermissionToEdit"],
    "isOwner": ["onIsOwner"],
    "locked": ["onLocked"],
    "isFirstRun": ["onIsFirstRun"],
    "isReady": ["onIsReady"],
    "answersResourceState": ["onAnswersResourcesStateChange"],
    "applicationFormResourceState": ["onApplicationFormResourceStateChange"],
    "questionPoolResourceState": ["onQuestionPoolResourcesStateChange"],
    "questionSetResourceState": ["onQuestionSetResourcesStateChange"],
    "shouldDisableIfIncomplete": ["onShouldDisableIfIncompleteChange"],
    "activeQuestionSet": ["onActiveQuestionSetChange"],
    "queue": ["onQueueChange"],
    "pendingProcesses": ["onPendingProcessesChange"]
  }; }
};
BpSdkNextButtonContainer.style = bpSdkNextButtonContainerCss;

export { BpSdkNextButtonContainer as bp_sdk_next_button_container };
